import {
    EncryptionAlgorithm,
    MatrixClient,
    MessageEvent,
    RustSdkCryptoStorageProvider,
    SimpleFsStorageProvider,
} from "matrix-bot-sdk";

import { StoreType } from "@matrix-org/matrix-sdk-crypto-nodejs";

import {
    logError,
    logInfo,
    readConfig,
    readMessages
} from './utils.js';

import {
    appendUserLikes,
    appendUserPictures,
    checkForMutualLike,
    eraseUser,
    eraseUserLikes,
    eraseUserMedia,
    getAmountOfUserPictures,
    getCurrentUserAction,
    getUserCurrentlyViewingProfile,
    setUserState
} from './db.js';

import { processRequest, showRandomProfileToUser, showNewLikes } from "./interactions.js";

import { db } from "./db.js";


const config = readConfig();
const messages = readMessages();

const homeserverUrl = config.homeserverURL;
const accessToken = config.token;
const maxAmountOfPhotoesPerUser = config.maxAmountOfPhotoesPerUser;

const crypto = new RustSdkCryptoStorageProvider("./bot_data/encryption_bot_sled", StoreType.Sled);
const storage = new SimpleFsStorageProvider("./bot_data/bot.json");

const client = new MatrixClient(homeserverUrl, accessToken, storage, crypto);

client.on("room.message", async (roomId, event) => {
    try {
        if (event.content?.msgtype !== 'm.text' && event.content?.msgtype !== 'm.image' && event.content?.msgtype !== 'm.video') return;
        if (event.sender === await client.getUserId()) return;

        let current_action = await getCurrentUserAction(roomId);
        let answer = event.content.body;

        switch (current_action) {
            case "wait_start":
                if (answer !== "!start") return;
                await setUserState(roomId, "country");
                await client.sendText(roomId, messages.setup.country);

                break;
            case "country":
                await processRequest(client, roomId, current_action, answer, 'city');
                break;
            case "city":
                await processRequest(client, roomId, current_action, answer, 'name');
                break;
            case "name":
                await processRequest(client, roomId, current_action, answer, 'age');
                break;
            case "age":
                answer = parseInt(answer)
                if (!answer) {
                    await client.sendText(roomId, messages.errors.didntunderstand);
                    return;
                }

                if (answer < 14) {
                    await client.sendText(roomId, messages.errors.tooyoung);
                    await client.leaveRoom(roomId);
                    await eraseUser(roomId);
                    return;
                }
                await processRequest(client, roomId, current_action, answer, 'sex');
                break;
            case "sex":
                answer = answer.toLowerCase().trim();
                if (answer.toLowerCase() != "male" && answer.toLowerCase() != "female") {
                    await client.sendText(roomId, messages.errors.twosexes);
                    return;
                }
                await processRequest(client, roomId, current_action, answer[0], 'interest');
                break;
            case "interest":
                answer = answer.toLowerCase().trim();
                if (answer != "male" && answer != "female" && answer != "both") {
                    await client.sendText(roomId, messages.errors.didntunderstand);
                    return;
                }
                await processRequest(client, roomId, current_action, answer[0], 'description');
                break;
            case "description":
                await processRequest(client, roomId, current_action, answer, 'pictures');
                break;
            case "pictures":
                if (event.content?.msgtype !== 'm.image' && event.content?.msgtype !== 'm.video') {
                    await client.sendText(roomId, messages.setup.done);
                    await setUserState(roomId, 'view_profiles');
                    await showRandomProfileToUser(client, roomId);
                } else {
                    let pictures_count = parseInt(await getAmountOfUserPictures(roomId));
                    if (pictures_count >= maxAmountOfPhotoesPerUser) {
                        await client.sendText(roomId, messages.errors.toomuch);
                        await setUserState(roomId, 'view_profiles');
                        await showRandomProfileToUser(client, roomId);
                    } else {
                        const message = new MessageEvent(event);
                        const fileEvent = new MessageEvent(message.raw);
                        const decrypted = await client.crypto.decryptMedia(fileEvent.content.file);
                        const mxc = await client.uploadContent(decrypted);
                        let type;
                        if (event.content.msgtype === "m.image") {
                            type = 'p';
                        } else if (event.content.msgtype === "m.video") {
                            type = 'v';
                        }
                        await appendUserPictures(roomId, mxc, type);
                        let pictures_count = await getAmountOfUserPictures(roomId);
                        if (pictures_count < maxAmountOfPhotoesPerUser) {
                            await client.sendText(roomId, messages.setup.more + String(maxAmountOfPhotoesPerUser - pictures_count));
                        } else {
                            await client.sendText(roomId, messages.setup.enough);
                            await client.sendText(roomId, messages.setup.done);
                            await setUserState(roomId, 'view_profiles');
                            await showRandomProfileToUser(client, roomId);
                        }
                    }
                }
                break;
            case "view_profiles":
                if (answer == '👍️' || answer == '❤️' || answer == '1') {
                    let currently_viewing = await getUserCurrentlyViewingProfile(roomId);
                    await appendUserLikes(roomId, currently_viewing);
                    let value = await checkForMutualLike(roomId, currently_viewing);
                    if (value) {
                        await client.sendText(roomId, messages.general.newlikes);
                        await client.sendText(currently_viewing, messages.general.newlikes);
                    }
                } else if (answer == '💌' || answer == '3') {
                    await client.sendText(roomId, messages.errors.notimplemented);
                } else if (answer == '🏠️' || answer == '4') {
                    await client.sendText(roomId, messages.general.menu);
                    await setUserState(roomId, 'menu')
                    return
                }

                await showRandomProfileToUser(client, roomId);
                break;
            case 'send_message':
                
                break;
            case 'menu':
                switch (answer) {
                    case '1':
                        await setUserState(roomId, 'view_profiles');
                        await showRandomProfileToUser(client, roomId);
                        break;
                    case '2':
                        await showNewLikes(client, roomId);
                        await client.sendText(roomId, messages.general.menu);
                        break;
                    case '3':
                        await setUserState(roomId, 'send_message');
                        await client.sendText(roomId, messages.errors.notimplemented);
                        break;
                    default:
                        await client.sendText(roomId, messages.errors.didntunderstand);
                        break;
                }
                break;
            default:
                await client.sendText(roomId, messages.errors.didntunderstand);
                return;
        }
    } catch (e) {
        logError(e);
    }
});

client.on("room.event", async (roomId, event) => {
    if (event.type === "m.room.member" && event.content?.membership === "leave") {
        await eraseUser(roomId);
        await eraseUserLikes(roomId);
        await eraseUserMedia(roomId);
        logInfo(`Bot has left a room with ID  ${roomId}`)
        client.leaveRoom(roomId);
    }
})

client.on("room.invite", async (roomId, event) => {
    try {
        await client.joinRoom(roomId);

        let members = await client.getAllRoomMembers(roomId)
        let isDM = members.length == 2 ? true : false;

        if (!isDM) {
            await client.sendText(roomId, messages.errors.notadm);
            await client.leaveRoom(roomId)
        }

        logInfo(`Bot has joined a room with ID ${roomId}`)

        let mx_id = event.sender

        let is_profile_exists = (await db.query("SELECT * FROM users WHERE room_id = $1", [roomId])).rowCount > 0;

        if (!is_profile_exists) {
            await db.query("INSERT INTO users(mx_id, room_id, current_action) VALUES ($1, $2, $3)", [mx_id, roomId, "wait_start"])

            await client.sendText(roomId, messages.general.welcome);
        } else {
            setUserState(roomId, 'view_profiles')
        }
    } catch (e) {
        logError(e)
    }

});

client.start().then(() => logInfo("Bot started!"));